import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;
import com.sunyard.security.sdf.AlgorithmEnum;
import com.sunyard.security.sdf.SDFJna;
import com.sunyard.security.util.ByteUtil;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class MutiThreads {

    private  PointerByReference phDeviceHandle = new PointerByReference();

    private static final byte[] data = new byte[16];
    private static final byte[] symKey = new byte[]{9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9};
    private static final String ip = "172.16.17.245";

    private static final int thread_max = 16;
    private static final int func_maxcall = 10000;

    @Before
    public void before(){
        // 打开设备
        int ret = SDFJna.INSTANCE.SDF_OpenDevice(
                phDeviceHandle,
                ByteUtil.safeStringBytes(ip),
                8889,
                3000,
                3000,
                0);

        if (ret != 0) {
            System.err.println("打开设备失败 错误码：" + String.format("0x%02X", ret));
            throw new RuntimeException(String.format("0x%02X", ret));
        }
    }


    @After
    public void after(){
        if ( null != phDeviceHandle.getValue() ) {
            SDFJna.INSTANCE.SDF_CloseDevice(phDeviceHandle.getValue());
        }
    }

    private void genSessionKey(PointerByReference phSessionHandle, PointerByReference phkeyHandle){

        int ret = SDFJna.INSTANCE.SDF_ImportKey(phSessionHandle.getValue(), symKey, symKey.length ,phkeyHandle);
        if (ret != 0) {
            throw new RuntimeException("genSessionKey " + String.format("0x%02X", ret));
        }
    }

    /**
     * 多线程加解密
     */
    @Test
    public void sm4_en(){
        System.out.println(" sm4 en 测试开始");

        Thread[] threads = new Thread[thread_max];
        for (  int i = 0; i < thread_max; i++ ) {
            threads[i] = new Thread(new Runnable() {
                @Override
                public void run() {

                    PointerByReference phSessionHandle = new PointerByReference();
                    try {
                        Pointer hDeviceHandle = phDeviceHandle.getValue();
                        // 打开会话
                        int ret = SDFJna.INSTANCE.SDF_OpenSession(hDeviceHandle, phSessionHandle);
                        if (ret != 0) {
                            System.err.println("打开设备失败 错误码：" + String.format("0x%02X", ret));
                            throw new RuntimeException(String.format("0x%02X", ret));
                        }

                        PointerByReference phkeyHandle = new PointerByReference();
                        genSessionKey(phSessionHandle, phkeyHandle);

                        for (int c= 0; c < func_maxcall; c++ ) {
                            byte[] enData = new byte[32];
                            IntByReference intByReference = new IntByReference();
                            ret = SDFJna.INSTANCE.SDF_Encrypt(phSessionHandle.getValue(), phkeyHandle.getValue(), AlgorithmEnum.SGD_SM4_ECB.getValue(), null, data, data.length, enData, intByReference);
                            if (ret != 0) {
                                throw new RuntimeException("加密失败 " + String.format("0x%02X", ret));
                            }
                        }
                    } finally {
                        if ( null !=  phSessionHandle.getValue() ) {
                            SDFJna.INSTANCE.SDF_CloseSession( phSessionHandle.getValue() );
                        }
                    }
                }
            });
        }

        long start = System.currentTimeMillis();
        for (Thread thread : threads) {
            thread.start();
        }

        for (Thread thread : threads) {
            try {
                thread.join();
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }

        long end = System.currentTimeMillis();
        long delta = end - start;
        double p = (double) thread_max * func_maxcall * 1000.0 / delta;
        System.out.println( " sm4 en " + p + "TPS");
    }

}
